import {
  Body,
  Controller,
  Get,
  Put,
  Query,
  Request,
  UseGuards,
} from '@nestjs/common';
import { AuthGuard } from '@nestjs/passport';
import {
  ApiBearerAuth,
  ApiBody,
  ApiOperation,
  ApiQuery,
  ApiTags,
} from '@nestjs/swagger';
import { Serealize } from '../interceptors/serialize.interceptor';
import { InstitucionDiaService } from './institucion-dia.service';
import { ValidarUsuarioService } from '../validar-usuario/validar-usuario.service';
import { Operador } from '../operador/entity/operador.entity';
import { Usuario } from '../usuario/entity/usuario.entity';
import { IdInstitucionDto } from '../dto/input/id-institucion.dto';
import { UpdateInstitucionDiaDto } from './dto/input/update.dto';
import { MessageOutputDto } from '../dto/output/message.dto';
import { InstitucionCarreaOutputDto } from './dto/output/institucion-carrera.dto';

@Controller('institucion-dia')
@ApiTags('institucion-dia')
export class InstitucionDiaController {
  constructor(
    private institucionDiaService: InstitucionDiaService,
    private validarUsuarioService: ValidarUsuarioService,
  ) {}

  @Serealize(InstitucionCarreaOutputDto)
  @Get()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description: 'Todas las instituciones día de un admin.',
  })
  @ApiBearerAuth('jwt')
  dias(@Request() req) {
    const admin: Operador = req.user.operador;

    this.validarUsuarioService.validarAdmin(admin);
    return this.institucionDiaService.findAllAdmin(admin);
  }

  @Serealize(InstitucionCarreaOutputDto)
  @Get('hoy')
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({ description: 'Infromación de día de hoy.' })
  @ApiBearerAuth('jwt')
  @ApiQuery({
    description: 'Id de la institución.',
    name: 'id_institucion',
    type: 'string',
  })
  hoy(@Request() req, @Query() query: IdInstitucionDto) {
    const usuario: Usuario = req.user.usuario;

    this.validarUsuarioService.validarUsuario(usuario);
    return this.institucionDiaService.hoy(parseInt(query.id_institucion));
  }

  @Serealize(MessageOutputDto)
  @Put()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description: 'Actualizar la información de un día de una institución.',
  })
  @ApiBearerAuth('jwt')
  @ApiBody({
    description:
      'Variables que necesita el endpoint. Las variables con "_" al inicio son opcionales.',
    examples: {
      ejemplo: {
        value: {
          id_institucion_dia: 217,
          _activo: true,
          _hora_extra: ':',
          _hora_fin: ':',
          _hora_inicio: ':',
          _hora_tope: ':',
        },
      },
    },
  })
  update(@Request() req, @Body() body: UpdateInstitucionDiaDto) {
    const admin: Operador = req.user.operador;

    this.validarUsuarioService.validarAdmin(admin);
    return this.institucionDiaService.update(admin, body);
  }
}
